# meds/cfg.py
#
#

"""
    default config objects containing default values for various services and plugins.

    >>> from meds.cfg import xmpp,rest,udp,cli,rss,main,stats,irc

    xmpp config

    >>> print(xmpp)
    {
        "cfg": "xmpp",
        "channel": "#meds",
        "nick": "meds",
        "owner": "",
        "port": 5222,
        "prefix": "cfg",
        "server": "localhost",
        "user": "meds@localhost",
        "username": "meds"
    }

    irc config

    >>> print(irc)
    {
        "cfg": "irc",
        "channel": "#meds",
        "ipv6": false,
        "nick": "meds",
        "owner": "",
        "port": 6667,
        "prefix": "cfg",
        "realname": "meds",
        "server": "localhost",
        "username": "meds"
    }

    main config

    >>> print(main)
    {
        "all": false,
        "args": [],
        "background": false,
        "cfg": "main",
        "colors": false,
        "debug": false,
        "default": [...],
        "exclude": "...",
        "init": "",
        "loglevel": "...",
        "modules": "",
        "nowait": true,
        "onerror": false,
        "openfire": false,
        "owner": "",
        "packages": [...],
        "path": false,
        "port": "...",
        "prefix": "cfg",
        "reboot": false,
        "resume": false,
        "scan": false,
        "services": false,
        "shell": false,
        "skip": "",
        "verbose": ...,
        "workdir": "/home/bart/.meds"
    }

    udp config

    >>> print(udp)
    {
        "cfg": "udp",
        "host": "localhost",
        "password": "boh",
        "port": 5500,
        "prefix": "cfg",
        "seed": "blablablablablaz"
    }

    rest config

    >>> print(rest)
    {
        "cfg": "rest",
        "hostname": "...",
        "port": 10102,
        "prefix": "cfg"
    }

    stats config

    >>> print(stats)
    {
        "cfg": "stats",
        "prefix": "cfg",
        "showurl": true
    }

    rss config

    >>> print(rss)
    {
        "cfg": "rss",
        "descriptions": [
            "officiel"
        ],
        "display_list": [
            "title",
            "link",
            "Date"
        ],
        "dosave": [],
        "ignore": [],
        "nosave": [],
        "showurl": false,
        "sleeptime": 600
    }

    cli config

    >>> print(cli)
    {
        "cfg": "cli",
        "welcome": "mogge!!"
    }

"""

from meds.utils.trace import get_strace
from meds.errors import EWORKDIR
from meds.object import Object

import logging
import socket
import time
import ast
import os

class Config(Object):

    def __getattr__(self, name):
        try: return super().__getattr__(name)
        except AttributeError: self[name] = ""
        return self[name]

    def load(self, name):
        from meds.core import storage
        obj = storage.last("cfg", name)
        if obj:
            logging.info("cfg on %s %s" % (name, obj))
            self.update(obj)

#:
xmpp = Config()
xmpp.prefix = "cfg"
xmpp.user = "meds@localhost"
xmpp.server = "localhost"
xmpp.username = "meds"
xmpp.channel = "#meds"
xmpp.nick = "meds"
xmpp.cfg = "xmpp"
xmpp.port = 5222
xmpp.owner = ""

#:
irc = Config()
irc.cfg = "irc"
irc.channel = "#meds"
irc.nick = "meds"
irc.ipv6 = False
irc.owner = ""
irc.port = 6667
irc.prefix = "cfg"
irc.realname = "meds"
irc.server = "localhost"
irc.username = "meds"

#:
main = Config()
main.path = False
main.prefix = "cfg"
main.colors = False
main.debug = False
main.background = False
main.openfire = False
main.shell = False
main.init = ""
main.workdir = os.path.expanduser("~/.meds")
main.loglevel = "error"
main.packages = ["meds.bots", "meds.plugs", "meds.cbs"]
main.modules = ""
main.default = ["aliases", "config"]
main.exclude = "test,xmpp"
main.services = False
main.cfg = "main"
main.args = []
main.all = False
main.scan = False
main.resume = False
main.reboot = False
main.verbose = False

#:
udp = Config()
udp.prefix = "cfg"
udp.cfg = "udp"
udp.host = "localhost"
udp.port = 5500
udp.password = "boh"
udp.seed = "blablablablablaz" # needs to be 16 chars wide

#:
rest = Config()
rest.prefix = "cfg"
rest.cfg = "rest"
rest.hostname = socket.getfqdn()
rest.port = 10102

#:
stats = Config()
stats.prefix = "cfg"
stats.cfg = "stats"
stats.showurl = True

#:
rss = Config()
rss.cfg = "rss"
rss.display_list = ["title", "link", "Date"]
rss.descriptions = ["officiel", ]
rss.sleeptime = 600
rss.ignore = []
rss.dosave = []
rss.nosave = []
rss.showurl = False

#:
cli = Config()
cli.welcome = "mogge!!"
cli.cfg = "cli"

#:
input = Config()
